/*******************************************************************************
*
* File Name: DATA_SYNC.c
*
* Beschreibung: Software Funktionen fuer die Datensynchorniesierung zwischen
*				Beschleunigungssensor und PC:
*				Es gibt ein Array, in dem die Daten der Register
*				zwischengespeichert werden:
*				die Registeradresse entspricht der Spalte
*				in der ersten Zeile sind die Daten des Registers,
*				in der zweitenn Zeile wird vermerkt, ob die Daten neu sind
*				und noch weiter verarbeitet werden muessen, hierfuer gibt es
*				folgende Vermerke:
*				- DATA_SYNC_NO_CHANGE Daten wurden nicht veraendert, keine Aktion noetig
*				- DATA_SYNC_SEND_TO_SENSOR die Daten haben sich am PC geaendert und muessen noch zum Sensor uebertragen werden
*				- DATA_SYNC_SEND_TO_USART die Daten des Sensors haben sich geaendert und muessen noch an den PC uebertragen werden
*
* Projekt: Uebung Mikrorechnertechnik II Versuch mit Beschleunigungssensor
*
* Sebastian Loercks 2013
*
*******************************************************************************/

#include "DATA_SYNC.h"


/*******************************************************************************
*Funktion       void DATA_SYNC_InitData_v(void)
*
*Beschreibung   Initialisiert das Array, in dem die Daten der Register 
				zwischengespeichert werden
				die Registeradresse entspricht der Spalte
				in der ersten Zeile sind die Daten des Registers,
				in der zweitenn Zeile wird vermerkt, ob die Daten neu sind
				und noch weiter verarbeitet werden muessen, hierfuer gibt es 
				folgende Vermerke:
					- DATA_SYNC_NO_CHANGE Daten wurden nicht veraendert, keine Aktion noetig
					- DATA_SYNC_SEND_TO_SENSOR die Daten haben sich am Pc geaendert und muessen noch zum Sensor uebertragen werden
					- DATA_SYNC_SEND_TO_USART die Daten des Sensors haben sich geaendert und muessen noch an den PC uebertragen werden
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_InitData(void)
{
	uint8_t i;				//Zaehlvariable fuer for-Schleife
		
	for (i=0; i<22; i++)
	{
		DataArray[0][i]=0;						//initialiesiert die Daten als 0
		DataArray[0][i]=DATA_SYNC_NO_CHANGE;	//am Anfang keine Synchronisation
	}
	
}


/*******************************************************************************
*Funktion       void DATA_SYNC_ReadAllRegister(void)
*
*Beschreibung   liest alle Register des Sensors aus und speichert die Daten,
				die sich geaendert haben in das DataArray mit dem Vermerk
				diese noch an den PC zu senden 
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_ReadAllRegister(void)
{
	uint8_t i;
	uint8_t Data;
	for (i=0; i<22;i++)
	{
		Data= TWI_ReciveData(SLAVE_ADRESS, i);			//Empfaengt Daten aus Register i vom Sensor ueber TWI
		if (Data!=DataArray[0][i])						//Sind es neue Daten?
		{
			DataArray[0][i]=Data;						//Daten werden in das DatenArray uebernommen
			DataArray[1][i]=DATA_SYNC_SEND_TO_USART;	//Vermerk, dass die Daten noch ueber USB an den PC uebermittelt werden muessen
		}	
		_delay_us(100);									//Wartezeit fuer TWI-Modul
	}
}

/*******************************************************************************
*Funktion       void DATA_SYNC_ReadOutputRegister(u8 Data)
*
*Beschreibung   liest alle OutputRegister des Sensors aus und speichert die Daten 
				in das DataArray mit dem Vermerk diese noch an den PC zu senden 
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_ReadOutputRegister(void)
{
	uint8_t i;
	uint8_t Data;
	for (i=2; i<8; i++)								//Fuer die Ausgaberegister
	{
		Data= TWI_ReciveData(SLAVE_ADRESS, i);		//Empfaengt Daten der Ausgaberegister ueber TWI
		DataArray[0][i]=Data;						//Daten werden in das DatenArray uebernommen
		DataArray[1][i]=DATA_SYNC_SEND_TO_USART;	//Vermerk, dass die Daten noch ueber USART an den PC uebermittelt werden muessen
		_delay_us(100);								//Wartezeit fuer TWI-Modul
	}
}

/*******************************************************************************
*Funktion       void DATA_SYNC_WriteNewDataToUsart(void)
*
*Beschreibung   sendet alle Daten aus dem DatenArray_u8 mit dem Vermerk 
				DATA_SYNC_SEND_TO_USART per USART und loescht den Vermerk
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_WriteNewDataToUsart(void)
{
	
	uint8_t i;
	for (i=0; i<22;i++)
	{
		if(DataArray[1][i]==DATA_SYNC_SEND_TO_USART)			//wenn geanderte Daten vorliegen und gesendet werden sollen
		{
			USART_SendData(i);									//sendet zunaechst die Registeradresse
			USART_SendData(DataArray[0][i]);					//sendet die Daten zur vorher gesendeten Adresse
			DataArray[1][i]=DATA_SYNC_NO_CHANGE;				//Vermerk zuruecksetzen
		}
	}
}

/*******************************************************************************
*Funktion       void DATA_SYNC_WriteOutputToUsart(void)
*
*Beschreibung   sendet alle Ausgangsregister (0x01 bis 0x06) unabhaenig vom 
				Vermerk an den PC
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_WriteOutputToUsart(void)
{
	
	uint8_t i;
	uint8_t Data;
	USART_SendData(166);			//sendet 166 -> Anfangscode fuer Ausgaberegisteruebermittlung
	for (i=7; i>1;i--)				//fuer alle Ausgaberegister
	{
		USART_SendData(DataArray[0][i]);	//sendet nur die Werte der Ausgaberegister
	}
}

/*******************************************************************************
*Funktion       void DATA_SYNC_WriteNewDataToSensor(void)
*
*Beschreibung   sendet alle Daten aus dem DatenArray mit dem Vermerk 
				DATA_SYNC_SEND_TO_SENSOR per TWI an den Sensor und loescht den Vermerk
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void DATA_SYNC_WriteNewDataToSensor(void)
{
	uint8_t i;
	for (i=0; i<22; i++)
	{
		if (DataArray[1][i]==DATA_SYNC_SEND_TO_SENSOR)			//Wenn Daten an den Sensor gesendet werden muessen
		{
			TWI_SendData(SLAVE_ADRESS, i, DataArray[0][i]);		//Sendet Daten ueber TWI an die richtige Registeradresse im Sensor
			DataArray[1][i]=DATA_SYNC_NO_CHANGE;				//Vermerk zuruecksetzen
		}
		_delay_us(100);											//Wartezeit fuer TWI-Modul
	}
}