/*******************************************************************************
*
* File Name: TWI.c
*
* Beschreibung: Software Funktionen fuer TWI
*
* Projekt: Mikrorechnertechnik II
*
*******************************************************************************/
#include "TWI.h"

/*******************************************************************************
*Funktion       void TWI_InitTWI_v(uint8_t BitRate, uint8_t PrescalerHighBit, uint8_t PrescalerLowBit)
*
*Beschreibung   Initialiesiert das TWI-Modul des Mikrocontrollers
*
*Parameter		BitRate: Bitrate die im Register TWBR eingestellt werden soll
*				PrescalerHighBit:	Wert der fuer TWPS1 eingestellt werden soll
*				PrescalerLowBit:	Wert der fuer TWPS0 eingestellt werden soll
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_InitTWI(uint8_t BitRate, uint8_t PrescalerHighBit, uint8_t PrescalerLowBit)
{
	// TWIFrequenz auswaehlen freq= F_CPU/(16+2(TWBR)*4^TWPS)
	TWBR=BitRate;												//Bitrate setzen
	TWSR=(PrescalerHighBit<<TWPS1)|(PrescalerLowBit<<TWPS0);    // Setzen der prescalar bits	
}


/*******************************************************************************
*Funktion       void TWI_StartCondition(void)
*
*Beschreibung   Sendet eine Startbedingung fuer eine TWI Kommunikation
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_StartCondition(void)
{
	// Setzt TWI interrupt flag zurueck, legt Startbedingung an SDA, Enable TWI
	TWCR= (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	while(!(TWCR & (1<<TWINT))); // wartet bis die Startbedingung gesendet ist
	while((TWSR & 0xF8)!= 0x08); // prueft acknowledgement
}

/*******************************************************************************
*Funktion       void TWI_RepeatedStartCondition(void)
*
*Beschreibung   Sendet eine erneute Startbedingung
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_RepeatedStartCondition(void)
{
	// Setzt TWI interrupt flag zurueck, legt Startbedingung an SDA, Enable TWI
	TWCR= (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	while(!(TWCR & (1<<TWINT))); // wartet bis die Startbedingung gesendet ist
	while((TWSR & 0xF8)!= 0x10); // prueft acknowledgement
}


/*******************************************************************************
*Funktion       void TWI_SendAddress(uint8_t Adress, uint8_t Direction)
*
*Beschreibung   Sendet die Slave Adresse, ob anschlieend gelesen oder geschriben werden soll und wartet ACK ab
*
*Parameter		Adress:	Adresse des Slaves im 8-Bit Format (letztes Bit egal)
*				Direction: Read=1; Write=0
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_SendAddress(uint8_t Adress, uint8_t Direction)
{
	if (Direction==0)
	{
		Adress&=0xFE;
	}
	else
	{
		Adress|=0x01;
	}
	TWDR=Adress;					// Addresse and Richtung senden
	TWCR=(1<<TWINT)|(1<<TWEN);		// Setzt TWI interrupt flag zurueck, Enable TWI
	while (!(TWCR & (1<<TWINT)));	// wartet bis TWDR uebertragen ist
	if (Direction==0)
	{
		while((TWSR & 0xF8)!= 0x18);  // prueft acknoledgement fuer lesen
	}
	else
	{
		while((TWSR & 0xF8)!= 0x40);  // prueft acknoledgement fuer schreiben
	}
	
}


/*******************************************************************************
*Funktion       void TWI_WriteData(uint8_t char Data)
*
*Beschreibung   Sendet 8-Bit Daten und wartet auf ACK
*
*Parameter		Data: 8-Bit Daten, die gesendet werden sollen
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_WriteData(uint8_t Data)
{
	TWDR=Data;						// legt Data ins Senderegister
	TWCR=(1<<TWINT)|(1<<TWEN);		// Setzt TWI interrupt flag zurueck, Enable TWI
	while (!(TWCR & (1<<TWINT)));	// wartet bis TWDR uebertragen ist
	while((TWSR & 0xF8) != 0x28);	// prueft acknoledgement
}

/*******************************************************************************
*Funktion       uint8_t TWI_ReadData(void)
*
*Beschreibung   Empfaengt 8-Bit Daten ueber TWI
*
*Parameter		keine
*
*Rueckgabe      8Bit Daten
*
*******************************************************************************/
uint8_t TWI_ReadData(void)
{
	uint8_t ReciveData;				//Variable fuer die empfangenden Daten
	TWCR=(1<<TWINT)|(1<<TWEN);		// Setzt TWI interrupt flag zurueck, Enable TWI
	while (!(TWCR & (1<<TWINT)));	// wartet bis das TWDR-Byte uebermittelt wurde
	while((TWSR & 0xF8) != 0x58);	// prueft acknoledgement
	ReciveData=TWDR;				// liest TWDR-Byte aus
	return (ReciveData);			// Rueckgabe der Daten
}

/*******************************************************************************
*Funktion       void TWI_StopCondition(void)
*
*Beschreibung   Sendet Stopp-Bedingung nach TWI-Kommunikation
*
*Parameter		keine
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_StopCondition(void)
{
	// Setzt TWI interrupt flag zurueck, legt Stoppbedingung an SDA, Enable TWI
	TWCR= (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	while(!(TWCR & (1<<TWSTO)));  // wartet bis Stoppbedingung uebermittelt wurde
}

/*******************************************************************************
*Funktion       void TWI_SendData(uint8_t SlaveAdress, uint8_t RegisterAdress, uint8_t Data)
*
*Beschreibung   Schreibt Daten ins das abgegebene Register des Slaves mit der angegebenen Adresse
*
*Parameter		SlaveAdress:	Adresse des Slaves, der angesprochen werden soll
*				Registeradress:	Register, in das geschrieben werden soll
*				Data:			Daten, die in das Register geschrieben werden sollen
*
*Rueckgabe      keine
*
*******************************************************************************/
void TWI_SendData(uint8_t SlaveAdress, uint8_t RegisterAdress, uint8_t Data)
{
	TWI_StartCondition();							//Start der TWI Kommunikation
	TWI_SendAddress(SlaveAdress, 0);				//Senden der Slave- und Registeradresse, write senden
	TWI_WriteData(RegisterAdress);					//Senden der Registeradresse
	TWI_WriteData(Data);							//Schreiben der zu sendenden Daten
	TWI_StopCondition();							//Beenden der TWI Kommunikation
}


/*******************************************************************************
*Funktion       uint8_t TWI_ReciveData(uint8_t SlaveAdress, uint8_t RegisterAdress)
*
*Beschreibung   Empfaengt Daten des Slaves, welche unter der angebenen Registeradresse zu finden sind
*
*Parameter		SlaveAdresse:	Adresse des angeschlossenen Slaves im 8-Bit Format
*				RegisterAdress: Adresse des Registers, das gelesen werden soll
*
*Rueckgabe      8-Bit Daten
*
*******************************************************************************/
uint8_t TWI_ReciveData(uint8_t SlaveAdress, uint8_t RegisterAdress)
{
	uint8_t ReceiveData;
	TWI_StartCondition();							//Start der TWI Kommunikation
	TWI_SendAddress(SlaveAdress, 0);				//Senden der Slave- und Registeradresse, write senden
	TWI_WriteData(RegisterAdress);					//Senden der Registeradresse
	TWI_RepeatedStartCondition();					//Erneuter Start der TWI Kommunikation
	TWI_SendAddress(SlaveAdress, 1);				//Senden der Slave- und Registeradresse, read senden
	//TWI_ReadData();									//Lesen der zu empfangenden Daten
	ReceiveData=TWI_ReadData();
	TWI_StopCondition();							//Beenden der TWI Kommunikation
	return (ReceiveData);										
}

//void TWI_SendData(uint8_t SlaveAdress, void* send_data, uint8_t send_bytes)
//{
	//if (send_bytes>0)
	//{
		//// Sende Start
		//TWCR=(1<<TWINT) | (1<<TWEN) | (1<<TWSTA);
		//while (!(TWCR & (1<<TWINT)));
		//uint8_t status=TWSR & 0xf8;
		//if (status != 0x08 && status != 0x10) goto error;
		//
		//// Sende Adresse (write mode)
		//TWDR=slave_address;
		//TWCR=(1<<TWINT) | (1<<TWEN);
		//while (!(TWCR & (1<<TWINT)));
		//if ((TWSR & 0xf8) != 0x18) goto error;
		//
		//// Sende Daten
		//while (send_bytes>0)
		//{
			//TWDR=*((uint8_t*)send_data);
			//TWCR=(1<<TWINT) | (1<<TWEN);
			//while (!(TWCR & (1<<TWINT)));
			//if ((TWSR & 0xf8) != 0x28) goto error;
			//send_data++;
			//send_bytes--;
		//}
	//}
	//
	//
//}
//
//uint8_t TWI_ReciveData(uint8_t SlaveAdressRead, uint8_t RegisterAdress, void* rcv_data, uint8_t rcv_bytes)
//{
    //if (rcv_bytes>0) 
	//{
	    //// Sende START
	    //TWCR=(1<<TWINT) | (1<<TWEN) | (1<<TWSTA);
	    //while (!(TWCR & (1<<TWINT)));
	    //uint8_t status=TWSR & 0xf8;
	    //if (status != 0x08 && status != 0x10) goto error;
	    //
	    //// Sende Adresse (read mode)
	    //TWDR=SlaveAdressRead;
	    //TWCR=(1<<TWINT) | (1<<TWEN);
	    //while (!(TWCR & (1<<TWINT)));
	    //if ((TWSR & 0xf8) != 0x40) goto error;
	    //
	    //// Empfange Daten
	    //while (rcv_bytes>0) 
		//{
		    //if (rcv_bytes==1) {
			    //// das letzte Byte nicht mit ACK quittieren
			    //TWCR=(1<<TWINT) | (1<<TWEN);
		    //}
		    //else
			//{
			    //// alle anderen Bytes mit ACK quittieren
			    //TWCR=(1<<TWINT) | (1<<TWEN) | (1<<TWEA);
		    //}
		    //while (!(TWCR & (1<<TWINT)));
		    //uint8_t status=TWSR & 0xf8;
		    //if (status!=0x50 && status != 0x58) goto error;
		    //*((uint8_t*)rcv_data)=TWDR;
		    //rcv_data++;
		    //rcv_bytes--;
	    //}
//
    //}
    //
    //// Sende STOP
    //TWCR=(1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
	//
    //error:
    //// Sende STOP
    //TWCR=(1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
    //return 0;
//}
/************************************************************************/
/*Register des TWI:
	- TWBR: Wert des TWI - Bitrate - Registers
	- TWPS: TWI Prescaler Bits, Wert der Prescalerbits im TWI - Status - Register
	- TWCR: TWI Control Register
		- TWINT: TWI Interrupt Flag, Lscht bisherige TWINTs
		- TWSTA: TWI START Condition Bit Senden des Startbefehls
		- TWSTO: TWI STOP Condition Bit
		- TWEN: TWI Enable Bit, Aktiviert den TWI
	- TWSR: TWI - Status - Register
	- TWDR: TWI Data Register, Datenregister des TWI
*/                                                                
/************************************************************************/